import { useLogger, requireModule, defineNuxtModule, createResolver, resolveModule, addPlugin, addAutoImport } from '@nuxt/kit';
import { withTrailingSlash } from 'ufo';
import { defu } from 'defu';
import { g as generateTokens } from './chunks/utils.mjs';
import { mkdir } from 'fs/promises';
import { existsSync } from 'fs';
import { resolve } from 'pathe';
import chalk from 'chalk';
import 'browser-style-dictionary/browser.js';

const name = "@nuxtjs/design-tokens-edge";
const version = "0.0.1-27629375.9c3b1e4";

const MODULE_DEFAULTS = {
  tokens: true
};
const logger = useLogger("design-tokens");
chalk.magentaBright(name);
const resolveConfig = (layer, key, configFile = `${key}.config`) => {
  const value = layer.config?.designTokens?.[key] || MODULE_DEFAULTS[key];
  let config = {};
  let filePath;
  if (typeof value === "boolean") {
    filePath = resolve(layer.cwd, configFile);
  } else if (typeof value === "string") {
    filePath = resolve(layer.cwd, value);
  } else if (typeof value === "object") {
    config = value;
  }
  if (filePath) {
    try {
      const _file = requireModule(filePath, { clearCache: true });
      if (_file) {
        config = _file;
      }
    } catch (_) {
    }
  }
  return { filePath, config };
};
const resolveTokens = (layers) => {
  const tokensFilePaths = [];
  let tokens = {};
  const splitLayer = (layer) => {
    if (layer.config?.designTokens?.tokens || MODULE_DEFAULTS.tokens) {
      const { config: layerTokens, filePath: _layerTokensFilePath } = resolveConfig(layer, "tokens", "tokens.config");
      if (_layerTokensFilePath) {
        tokensFilePaths.push(_layerTokensFilePath);
      }
      tokens = defu(tokens, layerTokens);
    }
  };
  for (const layer of layers) {
    splitLayer(layer);
  }
  return { tokensFilePaths, tokens };
};
const createTokensDir = async (path) => {
  if (!existsSync(path)) {
    await mkdir(path, { recursive: true });
  }
};

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "theme",
    compatibility: {
      nuxt: "^3.0.0-rc.4"
    }
  },
  defaults: MODULE_DEFAULTS,
  async setup(moduleOptions, nuxt) {
    nuxt.options.runtimeConfig.designTokens = {
      tokensFilePaths: []
    };
    const layers = nuxt.options._layers;
    const privateConfig = nuxt.options.runtimeConfig.designTokens;
    const tokensDir = withTrailingSlash(nuxt.options.buildDir + "/tokens");
    const refreshTokens = async (nitro) => {
      await createTokensDir(tokensDir);
      const { tokensFilePaths, tokens: tokens2 } = resolveTokens(layers);
      if (moduleOptions.tokens) {
        privateConfig.tokensFilePaths = tokensFilePaths;
      }
      if (nitro) {
        await nitro.storage.setItem("cache:design-tokens:tokens.json", tokens2);
      }
      return { tokens: tokens2, tokensFilePaths };
    };
    globalThis.__nuxtDesignTokensBuildDir__ = tokensDir;
    const { resolve: resolveTokensDir } = createResolver(tokensDir);
    privateConfig.tokensDir = tokensDir;
    const { tokens } = await refreshTokens();
    const { resolve: resolveRuntime } = createResolver(import.meta.url);
    const resolveRuntimeModule = (path) => resolveModule(path, { paths: resolveRuntime("./runtime") });
    nuxt.options.build.transpile = nuxt.options.build.transpile || [];
    nuxt.options.build.transpile.push(resolveRuntime("./runtime"));
    const buildTokens = async (nitro) => {
      try {
        const start = performance.now();
        const tokens2 = await nitro.storage.getItem("cache:design-tokens:tokens.json");
        await generateTokens(tokens2, tokensDir);
        const end = performance.now();
        logger.success(`Design Tokens built in ${start - end}ms`);
      } catch (e) {
        logger.error("Could not build tokens!");
        logger.error(e.message);
      }
    };
    nuxt.options.build.transpile.push("browser-style-dictionary/browser.js");
    nuxt.options.alias = nuxt.options.alias || {};
    nuxt.options.alias["#design-tokens"] = resolveTokensDir("index");
    nuxt.options.alias["#design-tokens/types"] = resolveTokensDir("types.d");
    nuxt.options.css = nuxt.options.css || [];
    nuxt.options.css = [...nuxt.options.css, resolveTokensDir("tokens.css")];
    nuxt.hook("prepare:types", (opts) => {
      opts.references.push({ path: resolveTokensDir("types.d") });
    });
    nuxt.hook("nitro:build:before", async (nitro) => {
      await refreshTokens(nitro);
      await buildTokens(nitro);
    });
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.handlers = nitroConfig.handlers || [];
      nitroConfig.handlers.push({
        method: "get",
        route: "/api/_design-tokens/tokens/generate",
        handler: resolveRuntimeModule("./server/api/tokens/generate")
      });
      nitroConfig.handlers.push({
        method: "all",
        route: "/api/_design-tokens/tokens",
        handler: resolveRuntimeModule("./server/api/tokens/index")
      });
      nitroConfig.prerender = nitroConfig.prerender || {};
      nitroConfig.prerender.routes = nitroConfig.prerender.routes || [];
      nitroConfig.prerender.routes.push("/api/_design-tokens/tokens");
      nitroConfig.prerender.routes.push("/api/_design-tokens/options");
      nitroConfig.bundledStorage = nitroConfig.bundledStorage || [];
      nitroConfig.bundledStorage.push("/cache/tokens");
      nitroConfig.bundledStorage.push("/tokens");
      nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
        inline: [
          "lodash",
          "browser-style-dictionary",
          "browser-style-dictionary/lib",
          "browser-style-dictionary/browser",
          resolveRuntime()
        ]
      });
      nitroConfig.alias["#design-tokens"] = resolveRuntime("./runtime/server/utils");
    });
    addPlugin({
      src: resolveRuntimeModule("./plugins/tokens")
    });
    addAutoImport({
      from: resolveRuntimeModule("./composables/tokens"),
      name: "$tokens",
      as: "$tokens"
    });
    addAutoImport({
      from: resolveRuntimeModule("./composables/tokens"),
      name: "$tokens",
      as: "$t"
    });
    await generateTokens(tokens, tokensDir, true);
    if (nuxt.options.dev) {
      nuxt.hook("nitro:init", (nitro) => {
        nuxt.hook("vite:serverCreated", (viteServer) => {
          nuxt.hook("builder:watch", async (_, path) => {
            const isTokenFile = moduleOptions.tokens ? privateConfig.tokensFilePaths.some((tokensFilePath) => tokensFilePath.includes(path.replace(".js", "")) || tokensFilePath.includes(path.replace(".ts", ""))) : false;
            if (isTokenFile) {
              const { tokens: tokens2 } = await refreshTokens(nitro);
              viteServer.ws.send({
                type: "custom",
                event: "nuxt-design-tokens:update",
                data: {
                  tokens: moduleOptions.tokens ? tokens2 : {}
                }
              });
              await buildTokens(nitro);
            }
          });
        });
      });
    } else {
      nuxt.hook("build:before", async () => await generateTokens(tokens, tokensDir, true));
    }
  }
});

export { module as default };
